@@ -11,17 +11,21 @@
 #include <unistd.h>
 
 int main(int argc, char *argv[]) {
-	(void)argc;
-
 	const char *usage = "Usage: seatd-launch [options] [--] command\n"
 			    "\n"
-			    "  -h	Show this help message\n"
-			    "  -v	Show the version number\n"
+			    "  -l <loglevel>	Log-level to pass to seatd\n"
+			    "  -h		Show this help message\n"
+			    "  -v		Show the version number\n"
 			    "\n";
 
 	int c;
-	while ((c = getopt(argc, argv, "vh")) != -1) {
+	char loglevel[16] = "info";
+	while ((c = getopt(argc, argv, "vhl:")) != -1) {
 		switch (c) {
+		case 'l':
+			strncpy(loglevel, optarg, sizeof loglevel);
+			loglevel[sizeof loglevel - 1] = '\0';
+			break;
 		case 'v':
 			printf("seatd-launch version %s\n", SEATD_VERSION);
 			return 0;
@@ -29,7 +33,7 @@ int main(int argc, char *argv[]) {
 			printf("%s", usage);
 			return 0;
 		case '?':
-			fprintf(stderr, "Try '%s -h' for more information.\n", argv[0]);
+			fprintf(stderr, "Try 'seatd-launch -h' for more information.\n");
 			return 1;
 		default:
 			abort();
@@ -42,41 +46,30 @@ int main(int argc, char *argv[]) {
 	}
 	char **command = &argv[optind];
 
-	char sockpath[32];
-	snprintf(sockpath, sizeof sockpath, "/tmp/seatd.%d.sock", getpid());
-
-	unlink(sockpath);
-
-	int fds[2];
-	if (pipe(fds) == -1) {
+	int readiness_pipe[2];
+	if (pipe(readiness_pipe) == -1) {
 		perror("Could not create pipe");
 		goto error;
 	}
 
+	// Start seatd
 	pid_t seatd_child = fork();
 	if (seatd_child == -1) {
 		perror("Could not fork seatd process");
 		goto error;
 	} else if (seatd_child == 0) {
-		close(fds[0]);
+		close(readiness_pipe[0]);
 
 		char pipebuf[16] = {0};
-		snprintf(pipebuf, sizeof pipebuf, "%d", fds[1]);
+		snprintf(pipebuf, sizeof pipebuf, "%d", readiness_pipe[1]);
 
-		char *env[2] = {NULL, NULL};
-		char loglevelbuf[32] = {0};
-		char *cur_loglevel = getenv("SEATD_LOGLEVEL");
-		if (cur_loglevel != NULL) {
-			snprintf(loglevelbuf, sizeof loglevelbuf, "SEATD_LOGLEVEL=%s", cur_loglevel);
-			env[0] = loglevelbuf;
-		}
-
-		char *command[] = {"seatd", "-n", pipebuf, "-s", sockpath, NULL};
+		char *env[1] = {NULL};
+		char *command[] = {"seatd", "-n", pipebuf, "-l", loglevel, "-z", NULL};
 		execve(SEATD_INSTALLPATH, command, env);
 		perror("Could not start seatd");
 		_exit(1);
 	}
-	close(fds[1]);
+	close(readiness_pipe[1]);
 
 	// Wait for seatd to be ready
 	char buf[1] = {0};
@@ -91,7 +84,7 @@ int main(int argc, char *argv[]) {
 		}
 
 		struct pollfd fd = {
-			.fd = fds[0],
+			.fd = readiness_pipe[0],
 			.events = POLLIN,
 		};
 
@@ -106,7 +99,7 @@ int main(int argc, char *argv[]) {
 		}
 
 		if (fd.revents & POLLIN) {
-			ssize_t n = read(fds[0], buf, 1);
+			ssize_t n = read(readiness_pipe[0], buf, 1);
 			if (n == -1 && errno != EINTR) {
 				perror("Could not read from pipe");
 				goto error_seatd;
@@ -115,17 +108,17 @@ int main(int argc, char *argv[]) {
 			}
 		}
 	}
-	close(fds[0]);
+	close(readiness_pipe[0]);
 
 	uid_t uid = getuid();
 	gid_t gid = getgid();
 
 	// Restrict access to the socket to just us
-	if (chown(sockpath, uid, gid) == -1) {
+	if (chown(SEATD_DEFAULTPATH, uid, gid) == -1) {
 		perror("Could not chown seatd socket");
 		goto error_seatd;
 	}
-	if (chmod(sockpath, 0700) == -1) {
+	if (chmod(SEATD_DEFAULTPATH, 0700) == -1) {
 		perror("Could not chmod socket");
 		goto error_seatd;
 	}
@@ -145,7 +138,7 @@ int main(int argc, char *argv[]) {
 		perror("Could not fork target process");
 		goto error_seatd;
 	} else if (child == 0) {
-		setenv("SEATD_SOCK", sockpath, 1);
+		setenv("SEATD_SOCK", SEATD_DEFAULTPATH, 1);
 		execvp(command[0], command);
 		perror("Could not start target");
 		_exit(1);
@@ -162,9 +155,6 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if (unlink(sockpath) != 0) {
-		perror("Could not unlink socket");
-	}
 	if (kill(seatd_child, SIGTERM) != 0) {
 		perror("Could not kill seatd");
 	}
@@ -178,7 +168,6 @@ int main(int argc, char *argv[]) {
 	}
 
 error_seatd:
-	unlink(sockpath);
 	kill(seatd_child, SIGTERM);
 error:
 	return 1;
