@@ -40,15 +40,15 @@ static int open_socket(const char *path, int uid, int gid) {
 		goto error;
 	}
 	if (uid != -1 || gid != -1) {
+		if (chmod(path, 0770) == -1) {
+			log_errorf("Could not chmod socket: %s", strerror(errno));
+			goto error;
+		}
 		if (chown(path, uid, gid) == -1) {
 			log_errorf("Could not chown socket to uid %d, gid %d: %s", uid, gid,
 				   strerror(errno));
 			goto error;
 		}
-		if (chmod(path, 0770) == -1) {
-			log_errorf("Could not chmod socket: %s", strerror(errno));
-			goto error;
-		}
 	}
 	return fd;
 error:
@@ -57,35 +57,23 @@ error:
 }
 
 int main(int argc, char *argv[]) {
-	char *loglevel = getenv("SEATD_LOGLEVEL");
-	enum libseat_log_level level = LIBSEAT_LOG_LEVEL_ERROR;
-	if (loglevel != NULL) {
-		if (strcmp(loglevel, "silent") == 0) {
-			level = LIBSEAT_LOG_LEVEL_SILENT;
-		} else if (strcmp(loglevel, "info") == 0) {
-			level = LIBSEAT_LOG_LEVEL_INFO;
-		} else if (strcmp(loglevel, "debug") == 0) {
-			level = LIBSEAT_LOG_LEVEL_DEBUG;
-		}
-	}
-	log_init();
-	libseat_set_log_level(level);
-
 	const char *usage = "Usage: seatd [options]\n"
 			    "\n"
 			    "  -h		Show this help message\n"
-			    "  -n <fd>          FD to notify readiness on\n"
+			    "  -n <fd>	FD to notify readiness on\n"
 			    "  -u <user>	User to own the seatd socket\n"
 			    "  -g <group>	Group to own the seatd socket\n"
-			    "  -s <path>	Where to create the seatd socket\n"
+			    "  -l <loglevel>	Log-level, one of debug, info, error or silent\n"
 			    "  -v		Show the version number\n"
 			    "\n";
 
 	int c;
 	int uid = -1, gid = -1;
 	int readiness = -1;
-	const char *socket_path = getenv("SEATD_SOCK");
-	while ((c = getopt(argc, argv, "vhn:s:g:u:")) != -1) {
+	bool unlink_existing_socket = true;
+	bool chown_socket = true;
+	enum libseat_log_level level = LIBSEAT_LOG_LEVEL_INFO;
+	while ((c = getopt(argc, argv, "vhn:g:u:l:z")) != -1) {
 		switch (c) {
 		case 'n':
 			readiness = atoi(optarg);
@@ -94,10 +82,11 @@ int main(int argc, char *argv[]) {
 				return 1;
 			}
 			break;
-		case 's':
-			socket_path = optarg;
-			break;
 		case 'u': {
+			if (!chown_socket) {
+				fprintf(stderr, "-u/-g and -z are mutually exclusive\n");
+				return 1;
+			}
 			struct passwd *pw = getpwnam(optarg);
 			if (pw == NULL) {
 				fprintf(stderr, "Could not find user by name '%s'.\n", optarg);
@@ -108,6 +97,10 @@ int main(int argc, char *argv[]) {
 			break;
 		}
 		case 'g': {
+			if (!chown_socket) {
+				fprintf(stderr, "-u/-g and -z are mutually exclusive\n");
+				return 1;
+			}
 			struct group *gr = getgrnam(optarg);
 			if (gr == NULL) {
 				fprintf(stderr, "Could not find group by name '%s'.\n", optarg);
@@ -117,6 +110,31 @@ int main(int argc, char *argv[]) {
 			}
 			break;
 		}
+		case 'l':
+			if (strcmp(optarg, "debug") == 0) {
+				level = LIBSEAT_LOG_LEVEL_DEBUG;
+			} else if (strcmp(optarg, "info") == 0) {
+				level = LIBSEAT_LOG_LEVEL_INFO;
+			} else if (strcmp(optarg, "error") == 0) {
+				level = LIBSEAT_LOG_LEVEL_ERROR;
+			} else if (strcmp(optarg, "silent") == 0) {
+				level = LIBSEAT_LOG_LEVEL_SILENT;
+			} else {
+				fprintf(stderr, "Invalid loglevel: %s\n", optarg);
+				return 1;
+			}
+			break;
+		case 'z':
+			// Running under seatd-launch. We do not unlink files
+			// to protect against multiple instances, and
+			// seatd-launch takes care of ownership.
+			if (uid != -1 || gid != -1) {
+				fprintf(stderr, "-u/-g and -z are mutually exclusive\n");
+				return 1;
+			}
+			unlink_existing_socket = false;
+			chown_socket = false;
+			break;
 		case 'v':
 			printf("seatd version %s\n", SEATD_VERSION);
 			return 0;
@@ -131,33 +149,46 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if (socket_path == NULL) {
-		socket_path = SEATD_DEFAULTPATH;
-		struct stat st;
-		if (stat(socket_path, &st) == 0) {
-			log_info("Removing leftover seatd socket");
-			unlink(socket_path);
+	log_init();
+	libseat_set_log_level(level);
+
+	struct stat st;
+	if (lstat(SEATD_DEFAULTPATH, &st) == 0) {
+		if (!S_ISSOCK(st.st_mode)) {
+			log_errorf("Non-socket file found at socket path %s, refusing to start",
+				   SEATD_DEFAULTPATH);
+			return 1;
+		} else if (!unlink_existing_socket) {
+			log_errorf("Socket file found at socket path %s, refusing to start",
+				   SEATD_DEFAULTPATH);
+			return 1;
+		} else {
+			// We only do this if the socket path is not user specified
+			log_infof("Removing leftover socket at %s", SEATD_DEFAULTPATH);
+			if (unlink(SEATD_DEFAULTPATH) == -1) {
+				log_errorf("Could not remove leftover socket: %s", strerror(errno));
+				return 1;
+			}
 		}
 	}
 
 	struct server server = {0};
 	if (server_init(&server) == -1) {
-		log_errorf("server_create failed: %s", strerror(errno));
+		log_errorf("server_init failed: %s", strerror(errno));
 		return 1;
 	}
 
-	int socket_fd = open_socket(socket_path, uid, gid);
+	int ret = 1;
+	int socket_fd = open_socket(SEATD_DEFAULTPATH, uid, gid);
 	if (socket_fd == -1) {
 		log_error("Could not create server socket");
-		server_finish(&server);
-		return 1;
+		goto error_server;
 	}
 	if (poller_add_fd(&server.poller, socket_fd, EVENT_READABLE, server_handle_connection,
 			  &server) == NULL) {
 		log_errorf("Could not add socket to poller: %s", strerror(errno));
 		close(socket_fd);
-		server_finish(&server);
-		return 1;
+		goto error_socket;
 	}
 
 	log_info("seatd started");
@@ -172,12 +203,18 @@ int main(int argc, char *argv[]) {
 	while (server.running) {
 		if (poller_poll(&server.poller) == -1) {
 			log_errorf("Poller failed: %s", strerror(errno));
-			return 1;
+			goto error_socket;
 		}
 	}
 
+	ret = 0;
+
+error_socket:
+	if (unlink(SEATD_DEFAULTPATH) == -1) {
+		log_errorf("Could not remove socket: %s", strerror(errno));
+	}
+error_server:
 	server_finish(&server);
-	unlink(socket_path);
 	log_info("seatd stopped");
-	return 0;
+	return ret;
 }
